package com.uracle.cloudpush;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import com.uracle.cloudpush.common.CommonDef;
import com.uracle.cloudpush.common.Utils;
import com.uracle.cloudpush.receiver.RegisterBroadcastReceiver;

import org.json.JSONException;
import org.json.JSONObject;

import m.client.push.library.PushManager;
import m.client.push.library.common.Logger;
import m.client.push.library.common.PushConstants;
import m.client.push.library.common.PushConstantsEx;
import m.client.push.library.utils.PushUtils;

public class LoginActivity extends Activity {

	private static final String TAG  = "LoginActivity";

	private BroadcastReceiver mLoginBroadcastReceiver;

	private TextView mLoginPhoneText;
	private EditText mUserNameEdit;
	private Button mLoginButton;
	ProgressDialog mProgressDialog = null;
	
	

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	    InitUI();
		

	}

	private void InitUI(){
		setContentView(R.layout.moe_login);

		// cuid
		mLoginPhoneText = (TextView) LoginActivity.this.findViewById(R.id.id);

		//mLoginPhoneText.setText(getString(R.string.demo_id));

		// cname
		mUserNameEdit = (EditText) LoginActivity.this.findViewById(R.id.pwd);
		//mUserNameEdit.setText(getString(R.string.demo_name));
		//mUserNameEdit.setFocusable(false);

		mLoginButton = (Button)findViewById(R.id.login);
		mLoginButton.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View view) {
				// TODO Auto-generated method stub
				onLogin(view);
			}
		});

	
		
	//	PushManager.getInstance().initPushServer(getApplicationContext());
		
	}
	
	
	

	public void onLogin(View view){

		if (mLoginPhoneText == null || mLoginPhoneText.getText().toString().length() == 0) {
			Toast.makeText(LoginActivity.this, getString(R.string.moe_not_support_phone), Toast.LENGTH_SHORT).show();
			return;
		}
		if (mUserNameEdit == null || mUserNameEdit.getText().toString().length() == 0) {
			Toast.makeText(LoginActivity.this, getString(R.string.moe_login_enter_pw), Toast.LENGTH_SHORT).show();
			return;
		}

		// FIX_IT
		JSONObject params = new JSONObject();
		try {
			params.put(PushConstants.KEY_CUID, mLoginPhoneText.getText().toString());
			params.put(PushConstants.KEY_CNAME, mUserNameEdit.getText().toString());

		} 
		catch (JSONException e) {
			e.printStackTrace();
		}
		Utils.showProgressBar(LoginActivity.this);
		
		PushManager.getInstance().registerServiceAndUser(getApplicationContext(), params);
	}


	@Override
	protected void onResume() {
		super.onResume();
		
		registerReceiver();
		Logger.i("onResume");

	}

	@Override
	protected void onPause() {
		super.onPause();

		unregisterReceiver();
	}

	@Override
	public void onBackPressed() {
		super.onBackPressed();

		System.exit(0);
	}

	public void registerReceiver() {
		if (mLoginBroadcastReceiver != null) {
			return;
		}

		IntentFilter intentFilter = new IntentFilter();
		intentFilter.addAction(LoginActivity.this.getPackageName()  + PushConstantsEx.ACTION_COMPLETED);

		mLoginBroadcastReceiver = new RegisterBroadcastReceiver() {

			@Override
			public void onReceive(Context context, Intent intent) {
				super.onReceive(context, intent);

				if(!PushUtils.checkValidationOfCompleted(intent, context)){
					return;
				}

				Utils.dissMissProgressBar();

				if (bundle.equals(PushConstantsEx.COMPLETE_BUNDLE.REG_SERVICE_AND_USER)) { 

					if (resultCode.equals(PushConstants.SUCCESS_RESULT_CODE)) {
						Toast.makeText(context, "로그인 성공!", Toast.LENGTH_SHORT).show();
						// 자동로그인 설정
						Utils.setConfigString(context, CommonDef.CONFIG_SAVE_LOGIN, "Y");
						
						Logger.e("CUID : " + mLoginPhoneText.getText().toString());
						Logger.e("CNAME : " + mUserNameEdit.getText().toString());
						mLoginPhoneText.getText().toString();
						PushUtils.setStringToStorage(CommonDef.CONFIG_USER_CUID, mLoginPhoneText.getText().toString(), getApplicationContext());
						PushUtils.setStringToStorage(CommonDef.CONFIG_USER_NAME,mUserNameEdit.getText().toString(), getApplicationContext());
						goPushMessageBox();
					}else {
						Toast.makeText(context, "서비스 등록 오류 error code: " + resultCode + " msg: " + resultMsg, Toast.LENGTH_SHORT).show();
					}
				}
			}
		};

		LocalBroadcastManager.getInstance(this).registerReceiver(mLoginBroadcastReceiver, intentFilter);
	}

	public void unregisterReceiver() {
		if (mLoginBroadcastReceiver != null) {
			LocalBroadcastManager.getInstance(this).unregisterReceiver(mLoginBroadcastReceiver);
			mLoginBroadcastReceiver = null;
		}
	}

	private void goPushMessageBox(){
		startActivity(new Intent(LoginActivity.this, PushListActivity.class));
		finish();
	}



	public void onExit(View v){
		onBackPressed();
	}

	



}
